package simplyconnect.mail.bounce.deliverystatus

import simplyconnect.mail.fields.Field
import simplyconnect.mail.fields.FieldParser

/**
 * User: david
 * Date: 27/03/13
 * Time: 5:58 PM
 */
class DeliveryStatusParser {
    DeliveryStatusNotification notification = new DeliveryStatusNotification()
    boolean isProcessingPerMessageField = true
    RecipientFields currentRecipientFields
    Field tempOriginalRecipientField
    DeliveryStatusNotification parse(String deliveryStatusContent) {
        List<Field> fields = new FieldParser().parse(deliveryStatusContent)

        fields.each { Field field ->
            handleField(field)
        }
        return notification
    }

    void handleField(Field field) {
        checkIfPerRecipientFields(field)
        if (isProcessingPerMessageField) {
            notification.addField(field)
        } else if (!cachingRecipientFields()){
            currentRecipientFields.addField(field)
        }
    }

    boolean cachingRecipientFields() {
        return tempOriginalRecipientField != null;
    }

    void checkIfPerRecipientFields(Field field) {
        // recipient fields always come after per message fields so as soon as we hit a per recipient field, switch mode
        // note that the Original-Recipient field is optional so we cache this and create new recipients based on the first
        // mandatory field - Final-Recipient
        if (field.name.toLowerCase() == "original-recipient") {
            isProcessingPerMessageField = false
            tempOriginalRecipientField = field
        }
        if (field.name.toLowerCase() == "final-recipient") {
            isProcessingPerMessageField = false
            currentRecipientFields = new RecipientFields()
            notification.addRecipient(currentRecipientFields)
            if (tempOriginalRecipientField) {
                currentRecipientFields.addField(tempOriginalRecipientField)
                tempOriginalRecipientField = null
            }
        }
    }
}
